import {AxiosInstance, AxiosRequestConfig} from "axios";
import {router} from "@/router";

const httpStatus = {
    200: "服务器成功返回请求的数据",
    201: "新建或修改数据成功",
    202: "一个请求已经进入后台排队（异步任务）",
    204: "删除数据成功",
    400: "发出的请求有错误，服务器没有进行新建或修改数据的操作",
    401: "用户没有权限（令牌、用户名、密码错误）",
    403: "用户得到授权，但是访问是被禁止的",
    404: "发出的请求针对的是不存在的记录，服务器没有进行操作",
    406: "请求的格式不可得",
    410: "请求的资源被永久删除，且不会再得到的",
    422: "当创建一个对象时，发生一个验证错误",
    500: "服务器发生错误，请检查服务器",
    502: "网关错误",
    503: "服务不可用，服务器暂时过载或维护",
    504: "网关超时",
};

/**
 * axios请求默认配置
 */
const axiosRequestDef: () => AxiosRequestConfig = () => {
    return {
        baseURL: '',
        method: 'post',
        headers: {},
        params: {},
        timeout: 60_000,
        withCredentials: true,
        responseType: 'json',
        responseEncoding: 'utf8',
        validateStatus: status => (status >= 200 && status < 400),
    };
};

/**
 * 自定义全局axios实例
 */
const customAxios: (axiosInstance: AxiosInstance) => AxiosInstance = axiosInstance => {
    // const notificationDef: any = {
    //     duration: 3500,
    //     position: "top-right",
    //     title: "错误",
    // };
    // 全局请求拦截
    axiosInstance.interceptors.request.use(
        request => request,
        error => {
            // window.$notify?.error({
            //     ...notificationDef,
            //     title: "请求发送失败",
            //     message: "发送请求给服务端失败，请检查电脑网络，再重试",
            // });
            return Promise.reject(error);
        },
    );
    // 全局拦截配置
    axiosInstance.interceptors.response.use(
        response => response,
        error => {
            const {response} = error;
            if (!error || !response) {
                // window.$notify?.error({
                //     ...notificationDef,
                //     message: "请求服务端异常",
                // });
                return Promise.reject(error);
            }
            if (response?.status === 401) {
                // 跳转到登录页面
                // window.$notify?.error({
                //     ...notificationDef,
                //     message: "当前用户未登录",
                // });
                // 跳转到登录页
                router.push({name: "login"}).finally();
                return Promise.reject(error);
            }
            const {data: {message, validMessageList}} = response;
            if (validMessageList) {
                // window.$notify?.error({
                //     ...notificationDef,
                //     message: "请求参数校验失败",
                // });
                return Promise.reject(error.response);
            } else if (message) {
                // const errorText = message ?? httpStatus[response.status] ?? "服务器异常";
                // window.$notify?.error({
                //     ...notificationDef,
                //     message: errorText,
                // });
            }
            return Promise.reject(error);
        }
    );
    return axiosInstance;
};

export default {
    httpStatus,
    axiosRequestDef,
    customAxios
}
