import {defineStore} from 'pinia'
import copy from '@/store/copy'
import lock from '@/store/lock'
import layer from '@/store/layer'
import compose from '@/store/compose'
import snapshot from '@/store/snapshot'
import {ruleStore} from "@/store/ruler-things";
import {getEditorMenuPos} from '@/utils/editorUtils'

export const globalStore = defineStore('global', {
    state: () => {
        return {
            ...compose.state,
            ...snapshot.state,
            ...copy.state,
            // 编辑器模式 edit preview
            editMode: 'edit',
            pageConfig: {
                // 页面方向 h 水平方向 v 垂直方向
                pageDirection: 'Vertical',
                // 页面长度：mm
                pageWidth: 210,
                // 页面高度：mm
                pageHeight: 290,
                // 模板标题
                title: '新建模板',
                // 模板编码
                code: 'TEMPLATE',
                // 默认缩放比例：100%
                // 页面边距 mm
                topMargin: 8,
                rightMargin: 8,
                bottomMargin: 8,
                leftMargin: 8,
                style: {
                    // 默认缩放比例：100%
                    scale: 1,
                    // 页面背景
                    background: '#ffffff',
                    // 默认字体颜色
                    color: '#000000',
                    // 默认字号
                    fontSize: 12,
                    // 默认字体
                    fontFamily: '微软雅黑',
                    // 默认行高
                    lineHeight: 1,
                }
            },
            // 是否在编辑器中，用于判断复制、粘贴组件时是否生效，如果在编辑器外，则无视这些操作
            isInEditor: false,
            componentData: [],
            elementBands: new Map(),
            dataSource: [],
            dataset: {},
            curComponent: null,
            curTableCell: null,
            curComponentIndex: null,
            // 点击画布时是否点中组件，主要用于取消选中组件用。
            // 如果没点中组件，并且在画布空白处弹起鼠标，则取消当前组件的选中状态
            isClickComponent: false,
            curTableSettingId: null,
            editorMenuInfo: {
                visible: false,
                style: {
                    top: 0,
                    left: 0
                }
            }
        }
    },
    actions: {
        ...layer.actions,
        ...compose.actions,
        ...snapshot.actions,
        ...copy.actions,
        ...lock.actions,
        setClickComponentStatus(status) {
            this.isClickComponent = status
        },

        setEditMode(state, mode) {
            state.editMode = mode
        },

        setInEditorStatus(status) {
            this.isInEditor = status
        },

        setCanvasStyle(state, style) {
            state.canvasStyleData = style
        },

        setCurComponent({component, index}) {
            this.curComponent = component
            this.curComponentIndex = index
            this.curTableCell = null
        },

        setCurTableCell({component}) {
            this.curTableCell = component
        },

        setPageSize({pageWidth, pageHeight}) {
            this.pageConfig.pageWidth = pageWidth
            this.pageConfig.pageHeight = pageHeight
            // console.log('w, h', pageWidth, pageHeight)
            ruleStore().setRect({pageWidth, pageHeight})
            ruleStore().setReDrawRuler()
        },

        togglePageDirection(pageDirection) {
            const pageConfig = this.pageConfig
            pageConfig.pageDirection = pageDirection
            ruleStore().setRect({
                pageWidth: pageDirection === 'Vertical' ? pageConfig.pageWidth : pageConfig.pageHeight,
                pageHeight: pageDirection === 'Vertical' ? pageConfig.pageHeight : pageConfig.pageWidth
            })
            ruleStore().setReDrawRuler()
        },

        setPageConfig(pageConfig) {
            this.pageConfig = pageConfig
            ruleStore().setRect({
                pageWidth: pageConfig.pageDirection === 'p' ? pageConfig.pageWidth : pageConfig.pageHeight,
                pageHeight: pageConfig.pageDirection === 'p' ? pageConfig.pageHeight : pageConfig.pageWidth
            })
            ruleStore().setReDrawRuler()
        },

        updateDataValue({data, key, value}) {
            data[key] = value
        },

        setPropValue({propValue, id}) {
            const componentData = this.componentData
            if (this.componentData.length) {
                let newComponentValue = null
                let newComponentIndex = null
                for (let i = 0; i < componentData.length; i++) {
                    if (componentData[i].id === id) {
                        newComponentIndex = i
                        newComponentValue = componentData[i]
                        newComponentValue.propValue = propValue
                    }
                }
                if (newComponentIndex !== null) {
                    this.componentData[newComponentIndex] = newComponentValue
                    this.recordSnapshot()
                }
            }
        },

        setBindValue({bindValue, id}) {
            const componentData = this.componentData
            if (componentData.length) {
                let newComponentValue = null
                let newComponentIndex = null
                for (let i = 0; i < componentData.length; i++) {
                    if (componentData[i].id === id) {
                        newComponentIndex = i
                        newComponentValue = componentData[i]
                        newComponentValue.bindValue = bindValue
                    }
                }
                if (newComponentIndex !== null) {
                    this.componentData[newComponentIndex] = newComponentValue
                    this.recordSnapshot()
                }
            }
        },

        setDataSource(state, dataSource) {
            state.dataSource = dataSource
        },

        setDataSet(state, dataSet) {
            state.dataSet = dataSet
        },

        setCurTableSettingId(state, id) {
            state.curTableSettingId = id
        },

        setShapeStyle({top, left, width, height, rotate}) {
            if (top) {
                this.curComponent.style.top = Math.round(top)
            }
            if (left) {
                this.curComponent.style.left = Math.round(left)
            }
            if (width) {
                this.curComponent.style.width = Math.round(width)
            }
            if (height) {
                this.curComponent.style.height = Math.round(height)
            }
            if (rotate) {
                this.curComponent.style.rotate = Math.round(rotate)
            }
        },

        setShapeSingleStyle({key, value}) {
            this.curComponent.style[key] = value
        },

        setComponentData(state, componentData = []) {
            state.componentData = componentData
        },
        getElementBandByCode(bandCode) {
            return this.elementBands.get(bandCode);
        },
        addElementBand(elementBand) {
            this.elementBands.set(elementBand.code, {
                band: {height: elementBand.height, splitType: 'Stretch'},
                elements: []
            })
        },
        changeBandHeight({code, height}) {
            const originElementBand = this.elementBands.get(code)
            if (originElementBand !== undefined) {
                originElementBand.band.height = height;
            }
        },
        deleteElementBand(elementBandCode) {
            if (elementBandCode !== undefined) {
                this.elementBands.delete(elementBandCode)
            }
        },
        addElement({bandCode, element, index}) {
            const elementBand = this.elementBands.get(bandCode);
            if (index !== undefined) {
                elementBand.elements.splice(index, 0, element)
            } else {
                elementBand.elements.push(element)
            }
        },
        deleteElement({elementBandCode, index}) {
            const elementBand = this.elementBands.get(elementBandCode);
            if (index === undefined) {
                index = this.curComponentIndex
            }
            if (index === this.curComponentIndex) {
                this.curComponentIndex = null
                this.curComponent = null
            }
            if (/\d/.test(index)) {
                elementBand.elements.splice(index, 1)
            }
        },
        addComponent({component, index}) {
            if (index !== undefined) {
                this.componentData.splice(index, 0, component)
            } else {
                this.componentData.push(component)
            }
        },
        deleteComponent(index) {
            if (index === undefined) {
                index = this.curComponentIndex
            }

            if (index === this.curComponentIndex) {
                this.curComponentIndex = null
                this.curComponent = null
            }

            if (/\d/.test(index)) {
                this.componentData.splice(index, 1)
            }
        },
        showEditorMenu({top, left}) {
            const editorMenuPos = getEditorMenuPos(top, left);
            this.editorMenuInfo = {
                visible: true,
                style: {
                    top: editorMenuPos.top,
                    left: editorMenuPos.left
                }
            };
        },
        hideEditorMenu() {
            this.editorMenuInfo = {
                visible: false,
                style: {
                    top: 0,
                    left: 0
                }
            };
        }
    },
    getters: {
        getPageConfig(state) {
            return state.pageConfig
        },
    }
})